@@ -254,7 +254,9 @@ import DataFrame.IO.CSV as CSV (
 import DataFrame.IO.Parquet as Parquet (readParquet)
 import DataFrame.IO.Unstable.CSV as UnstableCSV (
     fastReadCsvUnstable,
+    fastReadTsvUnstable,
     readCsvUnstable,
+    readTsvUnstable,
  )
 import DataFrame.Internal.Column as Column (
     Column,
@@ -279,6 +281,7 @@ import DataFrame.Internal.Row as Row (
     Any,
     Row,
     fromAny,
+    rowValue,
     toAny,
     toRowList,
     toRowVector,
